no more unversioned obsolete
[sliceimage.git] / build.sh
index 400dc5b..53df754 100755 (executable)
--- a/build.sh
+++ b/build.sh
@@ -1,19 +1,16 @@
 #!/bin/bash
 #
-# Builds all reference image for vservers.  To optimize for space it
-# will only build a complete base vserver reference image and then
+# Builds all reference image for slices.  To optimize for space it
+# will only build a complete base reference image and then
 # builds "stub" images that are just contain the additional files
 # and/or changes for a given reference image.  This is done to shrink
 # the RPM itself.  These will be pieced back together with the base
-# vserver reference image by an init script that is installed on the
-# node.
+# image by an init script that is installed on the node.
 #
 # Mark Huang <mlhuang@cs.princeton.edu>
 # Marc E. Fiuczynski <mef@cs.princeton.edu>
 # Copyright (C) 2004-2007 The Trustees of Princeton University
 #
-# $Id: build.sh,v 1.20 2007/09/06 20:41:23 faiyaza Exp $
-#
 
 PATH=/sbin:/bin:/usr/sbin:/usr/bin
 
@@ -37,65 +34,105 @@ pl_process_fedora_options $@
 shiftcount=$?
 shift $shiftcount
 
-# pldistro expected as $1 - defaults to planetlab
-pldistro=planetlab
-[ -n "$@" ] && pldistro=$1
+# pldistro expected as $1 
+pldistro=$1 ; shift
+# this comes from spec's slicefamily
+slicefamily=$1; shift
 
 # Do not tolerate errors
 set -e
 
-# Path's to the vserver references images and stubs
+# Path's to the reference images and stubs
+# This is inherited from util-vservers
 vrefdir=$PWD/vservers/.vref
-vstubdir=$PWD/vservers/.vstub
-
-# XXX: The vserver reference name should be passed in as an argument
-# rather than being hardcoded.
-vrefname=default
+vref=${vrefdir}/${slicefamily}
+# stubs are created in a subdir per slicefamily
+vstubdir=$PWD/vservers/.vstub/${slicefamily}
 
-# Make /vservers and default vserver reference image
-vref=${vrefdir}/${vrefname}
+# Create paths
 install -d -m 755 ${vref}
-
-# locate the packages and groups file
-pkgsfile=$(pl_locateDistroFile ../build/ ${pldistro} vserver.pkgs)
+install -d -m 755 ${vstubdir}
 
 # Some of the PlanetLab RPMs attempt to (re)start themselves in %post,
 # unless the installation is running inside the BootCD environment. We
 # would like to pretend that we are.
 export PL_BOOTCD=1
 
-# Populate image with vserver-reference packages
-pl_root_setup_chroot ${vref} -k -f $pkgsfile
+# Populate image with sliceimage packages
+pl_root_makedevs ${vref}
+# locate the packages and groups file
+pkgsfile=$(pl_locateDistroFile ../build/ ${pldistro} sliceimage.pkgs)
+pl_root_mkfedora ${vref} ${pldistro} $pkgsfile
+pl_root_tune_image ${vref}
 
-for systemvserver in ../build/config.${pldistro}/vserver-*.pkgs ; do
-    NAME=$(basename $systemvserver .pkgs | sed -e s,vserver-,,)
+systemslice_count=$(ls ../build/config.${pldistro}/sliceimage-*.pkgs 2> /dev/null | wc -l)
+[ $systemslice_count -gt 0 ] && for systemslice in $(ls ../build/config.${pldistro}/sliceimage-*.pkgs) ; do
+    NAME=$(basename $systemslice .pkgs | sed -e s,sliceimage-,,)
 
-    echo "--------START BUILDING system vserver ${NAME}: $(date)"
+    echo " * --------START BUILDING system sliceimage ${NAME}: $(date)"
 
     # "Parse" out the packages and groups for yum
-    systempackages=$(pl_getPackages ${pl_DISTRO_NAME} $systemvserver)
-    systemgroups=$(pl_getGroups ${pl_DISTRO_NAME} $systemvserver)
+    systempackages=$(pl_getPackages ${pl_DISTRO_NAME} $pldistro $systemslice)
+    systemgroups=$(pl_getGroups ${pl_DISTRO_NAME} $pldistro $systemslice)
+    systempips=$(pl_getPips ${pl_DISTRO_NAME} $pldistro $systemslice)
+    systemgems=$(pl_getGems ${pl_DISTRO_NAME} $pldistro $systemslice)
 
     vdir=${vstubdir}/${NAME}
     rm -rf ${vdir}/*
     install -d -m 755 ${vdir}
 
# Clone the base vserver reference to the system vserver reference
     (cd ${vref} && find . | cpio -m -d -u -p ${vdir})
     rm -f ${vdir}/var/lib/rpm/__db*
 
     # Communicate to the initialization script from which vref this stub was cloned
-    echo ${vrefname} > ${vdir}.cloned
-
-    # Install the system vserver specific packages
-    [ -n "$systempackages" ] && yum -c ${vdir}/etc/yum.conf --installroot=${vdir} -y install $systempackages
-    [ -n "$systemgroups" ] && yum -c ${vdir}/etc/yum.conf --installroot=${vdir} -y groupinstall $systemgroups
-
-    # Create a copy of the system vserver w/o the vserver reference files and make it smaller. 
+    echo ${slicefamily} > ${vdir}.cloned
+
+    # Install the system sliceimage specific packages
+    for yum_package in $systempackages; do
+       echo " * yum installing $yum_package"
+       yum -c ${vdir}/etc/mkfedora-yum.conf --installroot=${vdir} -y install $yum_package || \
+           echo " * WARNING image $systemslice - yum install $yum_package failed"
+    done
+    for group_plus in $systemgroups; do
+       yum_group=$(echo $group_plus | sed -e "s,+++, ,g")
+       echo " * yum groupinstalling $yum_group"
+        yum -c ${vdir}/etc/mkfedora-yum.conf --installroot=${vdir} -y groupinstall "$yum_group" || \
+           echo " * WARNING image $systemslice - yum groupinstall $yum_group failed"
+    done
+
+    # running pip or gem requires connectivity, and DNS
+    # so we expose the build-vm's /etc/resolv.conf to the current vdir 
+    mkdir -p ${vdir}/etc
+    cp /etc/resolv.conf ${vdir}/etc
+
+    # this requires pip to be available in sliceimage at that point
+    # fedora and debian -> python-pip
+    # on fedora the command is called pip-python (sigh.)
+    for pip in $systempips; do
+       echo " * pip installing $pip"
+       chroot ${vdir} pip -v install $pip || chroot ${vdir} pip-python -v $pip || \
+           echo " * WARNING image $systemslice - pip install $pip failed"
+    done
+
+    # same for gems; comes with ruby in fedora but ruby-devel is most likely a good thing
+    # we add --no-rdoc --no-ri to keep it low
+    for gem in $systemgems; do
+       echo " * gem installing $gem"
+       chroot ${vdir} gem install --no-rdoc --no-ri $gem || \
+           echo " * WARNING image $systemslice - gem install $gem failed"
+    done
+
+    # search e.g. sliceimage-planetflow.post in config.<pldistro> or in config.planetlab otherwise
+    postfile=$(pl_locateDistroFile ../build/ ${pldistro} sliceimage-${NAME}.post || : )
+
+    [ -f $postfile ] && /bin/bash $postfile ${vdir} || :
+
+    # Create a copy of the system sliceimage w/o the sliceimage reference files and make it smaller. 
     # This is a three step process:
 
     # step 1: clean out yum cache to reduce space requirements
-    yum -c ${vdir}/etc/yum.conf --installroot=${vdir} -y clean all
+    yum -c ${vdir}/etc/mkfedora-yum.conf --installroot=${vdir} -y clean all
 
     # step 2: figure out the new/changed files in ${vdir} vs. ${vref} and compute ${vdir}.changes
     rsync -anv ${vdir}/ ${vref}/ > ${vdir}.changes
@@ -117,7 +154,22 @@ for systemvserver in ../build/config.${pldistro}/vserver-*.pkgs ; do
     rm -rf ${vdir}
     rm -f  ${vdir}.changes
     mv ${vdir}-tmp ${vdir}
-    echo "--------DONE BUILDING system vserver ${NAME}: $(date)"
+
+    # cleanup yum remainings
+    rm -rf ${vdir}/build ${vdir}/longbuildroot
+
+    echo " * --------DONE BUILDING system sliceimage ${NAME}: $(date)"
 done
 
+# search sliceimage.post in config.<pldistro> or in config.planetlab otherwise
+postfile=$(pl_locateDistroFile ../build/ ${pldistro} sliceimage.post)
+
+[ -f $postfile ] && /bin/bash $postfile ${vref} || :
+
+# fix sudoers config
+[ -f ${vref}/etc/sudoers ] && echo -e "\nDefaults\tlogfile=/var/log/sudo\n" >> ${vref}/etc/sudoers
+
+# cleanup yum remainings
+rm -rf ${vref}/build ${vref}/longbuildroot
+
 exit 0
@@ -37,65 +34,105 @@ pl_process_fedora_options $@
     install -d -m 755 ${vdir}
 
-    # Clone the base vserver reference to the system vserver reference
+    # Clone the base sliceimage reference to the system sliceimage reference
     (cd ${vref} && find . | cpio -m -d -u -p ${vdir})
     rm -f ${vdir}/var/lib/rpm/__db*